@@ -25,24 +25,7 @@ import (
 	"strings"
 )
 
-// Global application strings
-const (
-	shortUsage = "A tool for managing Linux filesystem encryption"
-
-	apache2GoogleCopyright = `Copyright 2017 Google, Inc.
-
-  Licensed under the Apache License, Version 2.0 (the "License");
-  you may not use this file except in compliance with the License.
-  You may obtain a copy of the License at
-
-      http://www.apache.org/licenses/LICENSE-2.0
-
-  Unless required by applicable law or agreed to in writing, software
-  distributed under the License is distributed on an "AS IS" BASIS,
-  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-  See the License for the specific language governing permissions and
-  limitations under the License.`
-)
+const shortUsage = "A tool for managing Linux filesystem encryption"
 
 // Argument usage strings
 const (
@@ -103,23 +86,6 @@ Options:
 {{range .VisibleFlags}}{{.}}
 
 {{end}}{{end}}`
-
-	// Additional info, used with "fscrypt version"
-	versionInfoTemplate = `{{.HelpName}} - {{.Usage}}
-
-{{if .Version}}Version:
-` + indent + `{{.Version}}
-
-{{end}}{{if not .Compiled.IsZero}}Compiled:
-` + indent + `{{.Compiled}}
-
-{{end}}{{if len .Authors}}Author{{with $length := len .Authors}}{{if ne 1 $length}}s{{end}}{{end}}:{{range .Authors}}
-` + indent + `{{.}}{{end}}
-
-{{end}}{{if .Copyright}}Copyright:
-` + indent + `{{.Copyright}}
-
-{{end}}`
 )
 
 // Add words to this map to have pluralize support them.
